/*
******************************************************************************
*
*      File             : tfo_main.c
*      Version          : 2.0.0
*      Date             : 27 February 2001
*      Author           : Jochen Junkawitsch
*      Description      : source file for TFO decision algorithm
*
*
******************************************************************************
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "tfo_decision.h"






/* ------------------------------------------------------------------- */



/* convert a string (example "-X--XX-X") to a CodecSet */
CodecSet s2cs(char* s)
{
  int k;
  CodecSet m, cs = 0;
  
  if (strlen(s) != 8)
    {fprintf(stderr,"ERROR: reading input\n");exit(0);}
  
  for (k=0, m=mode_122; k<8; k++, m >>= 1)
    if (s[k] == 'X')
      cs |= m;
  
  return cs;
}



/* read input Data from stdin */
int getInputData(Data* data)
{
  char buffer[255];

  char lscs[255], dscs[255];
  char lacs[255], dacs[255];
  int lmacs, dmacs;
  char lType[255], dType[255];
  char lOM, dOM;
  
  data->cacs = data->oacs = data->cscs = 0;

  
  if (!fgets(buffer, 255, stdin))
	  return 0;

  if (sscanf(buffer, "%s %s %d %s %c %s %s %d %s %c",
			 lscs, lacs, &lmacs, lType, &lOM,
			 dscs, dacs, &dmacs, dType, &dOM) != 10)
  {fprintf(stderr,"ERROR: reading input\n");exit(0);}
  

  data->lscs  = s2cs(lscs);
  data->lacs  = s2cs(lacs);
  if (lmacs > 0 && lmacs <= 8)
	  data->lmacs = lmacs;
  else
  {fprintf(stderr,"ERROR: reading input\n");exit(0);}
  
  if (!strcmp(lType, "FR_AMR"))
	  data->lType = FR_AMR;
  else if (!strcmp(lType, "HR_AMR"))
	  data->lType = HR_AMR;
  else if (!strcmp(lType, "UMTS_AMR"))
	  data->lType = UMTS_AMR;
  else if (!strcmp(lType, "UMTS_AMR_2"))
	  data->lType = UMTS_AMR_2;
  else if (!strcmp(lType, "GSM_EFR"))
	  data->lType = GSM_EFR;
  else if (!strcmp(lType, "GSM_FR"))
	  data->lType = GSM_FR;
  else if (!strcmp(lType, "GSM_HR"))
	  data->lType = GSM_HR;
  else
  {fprintf(stderr,"ERROR: reading input\n");exit(0);}
  
  if (lOM == 'y')
	  data->lOM = 1;
  else if (lOM == 'n')
	  data->lOM = 0;
  else
  {fprintf(stderr,"ERROR: reading input\n");exit(0);}

  data->dscs  = s2cs(dscs);
  data->dacs  = s2cs(dacs);
  if (dmacs > 0 && dmacs <= 8)
	  data->dmacs = dmacs;
  else
  {fprintf(stderr,"ERROR: reading input\n");exit(0);}

  if (!strcmp(dType, "FR_AMR"))
	  data->dType = FR_AMR;
  else if (!strcmp(dType, "HR_AMR"))
	  data->dType = HR_AMR;
  else if (!strcmp(dType, "UMTS_AMR"))
	  data->dType = UMTS_AMR;
  else if (!strcmp(dType, "UMTS_AMR_2"))
	  data->dType = UMTS_AMR_2;
  else if (!strcmp(dType, "GSM_EFR"))
	  data->dType = GSM_EFR;
  else if (!strcmp(dType, "GSM_FR"))
	  data->dType = GSM_FR;
  else if (!strcmp(dType, "GSM_HR"))
	  data->dType = GSM_HR;
  else
  {fprintf(stderr,"ERROR: reading input\n");exit(0);}

  if (dOM == 'y')
	  data->dOM = 1;
  else if (dOM == 'n')
	  data->dOM = 0;
  else
  {fprintf(stderr,"ERROR: reading input\n");exit(0);}

  /* check some settings */

  if (data->lType != FR_AMR && data->lType != HR_AMR &&
	  data->lType != UMTS_AMR && data->lType != UMTS_AMR_2)
    data->lOM = data->lscs = data->lacs =  data->lmacs = 0;

  if (data->dType != FR_AMR && data->dType != HR_AMR &&
	  data->dType != UMTS_AMR && data->dType != UMTS_AMR_2)
    data->dOM = data->dscs = data->dacs = data->dmacs = 0;

  if ((data->lacs & data->lscs) != data->lacs ||
      (data->dacs & data->dscs) != data->dacs)
    {fprintf(stderr,"ERROR: ACS not in SCS\n");exit(0);}

  if (data->lmacs < nModes(data->lacs) ||
      data->dmacs < nModes(data->dacs))
    {fprintf(stderr,"ERROR: MACS < #ACS\n");exit(0);}
  
  if (data->lType == HR_AMR)
    {data->lscs &= 0x3f; data->lacs &= 0x3f;}

  if (data->dType == HR_AMR)
    {data->dscs &= 0x3f; data->dacs &= 0x3f;}

  return 1;  
}




/* print Data to stdout */
int printData(Data* data)
{
  CodecSet m;

  printf("\n\n------------------------------------------------------------------");
  printf("\n\n\t%s\t\t\t\t\t%s\n", codecName(data->lType), codecName(data->dType));
  printf("\tMACS = %d\t\t\t\tMACS = %d\n", data->lmacs, data->dmacs);
  printf("\tOM   = %s\t\t\t\tOM   = %s\n", data->lOM ? "yes" : "no", data->dOM ? "yes" : "no");
  
  printf("\n\tSCS\tACS\tIACS\tOACS\tCSCS\tACS\tSCS\n");

  for (m=mode_122; m; m>>=1)
  {
	  printf("%s\t%c\t%c\t%c\t%c\t%c\t%c\t%c\n",
			 modeName((CodecMode)m),
			 data->lscs & m ? 'X' : '-',
			 data->lacs & m ? 'X' : '-',
			 data->cacs & m ? 'X' : '-',
			 data->oacs & m ? 'X' : '-',
			 data->cscs & m ? 'X' : '-',
			 data->dacs & m ? 'X' : '-',
			 data->dscs & m ? 'X' : '-');
  }
    
  printf("\n");
  return 0;
}





/* -------------------------------------------------------------------------------
                                      main
   ------------------------------------------------------------------------------- */

int main()
{
  Data a;
  Data* data = &a;
  

  
  while (getInputData(data))
  {
    determine_IACS(data);
    determine_OACS(data);
    printData(data);
    tfo_decision_main(data);
  }
  
    
  return 0;
}

